package opengles2.tutorial2;

import android.opengl.GLES20;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import javax.microedition.khronos.opengles.GL10;

public class ObjTriangle {
    private FloatBuffer vertexBuffer;
    private ShortBuffer indexBuffer;

    private final int COORDS_PER_VERTEX = 2;
    private final int vertexStride = COORDS_PER_VERTEX * 4;
    private float[] vertices = {
            -0.5f, -0.5f,
            0.5f, -0.5f,
            0.0f, 0.5f,
    };

    private short[] indices = {
            0, 1, 2
    };

    // Color for all vertices (Red).
    private float[] color = {
            1.0f, 0.0f, 0.0f, 1.0f
    };

    private final String vertexShaderCode =
            "uniform mat4 uMVPMatrix;" +
            "attribute vec4 vPosition;" +
            "void main() {" +
            "  gl_Position = uMVPMatrix * vPosition;" +
            "}";

    private final String fragmentShaderCode =
            "precision mediump float;" +
            "uniform vec4 vColor;" +
            "void main() {" +
            "  gl_FragColor = vColor;" +
            "}";

    private int mProgram;

    public ObjTriangle() {
        // Set vertex buffer from data.
        ByteBuffer vbb = ByteBuffer.allocateDirect(this.vertices.length * 4);
        vbb.order(ByteOrder.nativeOrder());
        this.vertexBuffer = vbb.asFloatBuffer();
        this.vertexBuffer.put(this.vertices);
        this.vertexBuffer.position(0);

        // Set index buffer from data.
        ByteBuffer ibb = ByteBuffer.allocateDirect(this.indices.length * 2);
        ibb.order(ByteOrder.nativeOrder());
        this.indexBuffer = ibb.asShortBuffer();
        this.indexBuffer.put(this.indices);
        this.indexBuffer.position(0);
    }
    public void SetProgramShader() {
        // Load shaders.
        int vertexShader = MainRenderer.LoadShader(GLES20.GL_VERTEX_SHADER, this.vertexShaderCode);
        int fragmentShader = MainRenderer.LoadShader(GLES20.GL_FRAGMENT_SHADER, this.fragmentShaderCode);

        // Create empty OpenGL ES Program.
        this.mProgram = GLES20.glCreateProgram();
        // Add the vertex shader to program.
        GLES20.glAttachShader(this.mProgram, vertexShader);
        // Add the fragment shader to program.
        GLES20.glAttachShader(this.mProgram, fragmentShader);
        // Creates OpenGL ES program executables.
        GLES20.glLinkProgram(this.mProgram);
    }

    private int positionHandle;
    private int colorHandle;
    private int modelHandle;

    public void DrawTriangle(float[] modelMatrix) {
        // Add program to OpenGL ES environment.
        GLES20.glUseProgram(this.mProgram);

        // Get handle to vertex shader's vPosition member.
        this.positionHandle = GLES20.glGetAttribLocation(this.mProgram, "vPosition");
        // Enable a handle to the triangle vertices.
        GLES20.glEnableVertexAttribArray(this.positionHandle);
        // Prepare the triangle coordinate data.
        GLES20.glVertexAttribPointer(this.positionHandle, COORDS_PER_VERTEX, GLES20.GL_FLOAT, false, this.vertexStride, this.vertexBuffer);

        // Get handle to fragment shader's vColor member.
        this.colorHandle = GLES20.glGetUniformLocation(this.mProgram, "vColor");
        // Set color for drawing the triangle.
        GLES20.glUniform4fv(this.colorHandle, 1, this.color, 0);

        // Get handle to shape's transformation matrix.
        this.modelHandle = GLES20.glGetUniformLocation(this.mProgram, "uMVPMatrix");
        // Pass the projection and view transformation to the shader.
        GLES20.glUniformMatrix4fv(this.modelHandle, 1, false, modelMatrix, 0);

        // Draw the triangle.
        GLES20.glDrawElements(GL10.GL_TRIANGLES, this.indices.length, GL10.GL_UNSIGNED_SHORT, this.indexBuffer);

        // Disable vertex array.
        GLES20.glDisableVertexAttribArray(this.positionHandle);
    }

}
